/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2024/02/04 11:56:58
 */

import { Point, isInLine } from '../math.js'

// Remove points, that looks like straight line
export function simplify(contours, accuracy) {
  return contours.map(function (contour) {
    let i, curr, prev, next
    let p, pPrev, pNext

    // run from the end, to simplify array elements removal
    for (i = contour.length - 2; i > 1; i--) {
      prev = contour[i - 1]
      next = contour[i + 1]
      curr = contour[i]

      // skip point (both oncurve & offcurve),
      // if [prev,next] is straight line
      if (prev.onCurve && next.onCurve) {
        p = new Point(curr.x, curr.y)
        pPrev = new Point(prev.x, prev.y)
        pNext = new Point(next.x, next.y)
        if (isInLine(pPrev, p, pNext, accuracy)) {
          contour.splice(i, 1)
        }
      }
    }
    return contour
  })
}

// Remove interpolateable oncurve points
// Those should be in the middle of nebor offcurve points
export function interpolate(contours, accuracy) {
  return contours.map(function (contour) {
    let resContour = []

    contour.forEach(function (point, idx) {
      // Never skip first and last points
      if (idx === 0 || idx === contour.length - 1) {
        resContour.push(point)
        return
      }

      let prev = contour[idx - 1]
      let next = contour[idx + 1]

      let p, pPrev, pNext

      // skip interpolateable oncurve points (if exactly between previous and next offcurves)
      if (!prev.onCurve && point.onCurve && !next.onCurve) {
        p = new Point(point.x, point.y)
        pPrev = new Point(prev.x, prev.y)
        pNext = new Point(next.x, next.y)
        if (pPrev.add(pNext).div(2).sub(p).dist() < accuracy) {
          return
        }
      }
      // keep the rest
      resContour.push(point)
    })
    return resContour
  })
}

export function roundPoints(contours) {
  return contours.map(contour =>
    contour.map(p => ({
      x: Math.round(p.x),
      y: Math.round(p.y),
      onCurve: p.onCurve
    }))
  )
}

// Remove closing point if it is the same as first point of contour.
// TTF doesn't need this point when drawing contours.
export function removeClosingReturnPoints(contours) {
  return contours.map(function (contour) {
    let length = contour.length

    if (
      length > 1 &&
      contour[0].x === contour[length - 1].x &&
      contour[0].y === contour[length - 1].y
    ) {
      contour.splice(length - 1)
    }
    return contour
  })
}

export function toRelative(contours) {
  let prevPoint = { x: 0, y: 0 }
  let resContours = []

  contours.forEach(function (contour) {
    let resContour = []
    resContours.push(resContour)
    contour.forEach(function (point) {
      resContour.push({
        x: point.x - prevPoint.x,
        y: point.y - prevPoint.y,
        onCurve: point.onCurve
      })
      prevPoint = point
    })
  })
  return resContours
}

export function identifier(string, littleEndian) {
  let result = 0

  for (let i = 0; i < string.length; i++) {
    result = result << 8
    let index = littleEndian ? string.length - i - 1 : i

    result += string.charCodeAt(index)
  }

  return result
}
